@@ -3,7 +3,11 @@ testthat::test_that(
   code = {
     mr <- c(100, 105)
     db <- GenerateMetaboliteSQLiteDB(dbfile = NULL, ionization = "APCI", mass_range = mr, ncores = 1)
-    testthat::expect_equal(nrow(db), 140)
+    if (requireNamespace("Rdisop", quietly = TRUE)) {
+      testthat::expect_equal(nrow(db), 140)
+    } else {
+      testthat::expect_null(db)
+    }
   }
 )
 
@@ -12,6 +16,10 @@ testthat::test_that(
   code = {
     mr <- c(100.6, 101.6)
     db <- GenerateMetaboliteSQLiteDB(dbfile = NULL, ionization = "ESI", mass_range = mr, ncores = 1)
-    testthat::expect_equal(nrow(db), 170)
+    if (requireNamespace("Rdisop", quietly = TRUE)) {
+      testthat::expect_equal(nrow(db), 170)
+    } else {
+      testthat::expect_null(db)
+    }
   }
 )
